
from django.shortcuts import render

from basement.utils import decorate


def render_view(template_name, renderer=render, decorators=None):

    def decorator(view_func):

        def wrapper(request, *args, **kwargs):

            context = decorate(view_func, decorators)(request, *args, **kwargs)

            return renderer(request, template_name, context)

        return wrapper

    return decorator




from basement import http
from basement.views._base_view import BaseView


class FormView(BaseView):

    def __init__(
            self,
            form_class,
            form_factory=None,
            response_class=http.JsonResponse,
            response_factory=None,
            **kwargs):

        self._form_class = form_class

        if form_factory is not None:
            self._form_factory = form_factory

        self._response_class = response_class

        if response_factory is not None:
            self._response_factory = response_factory

        super(FormView, self).__init__(**kwargs)

    def get(self, request, **url_kwargs):

        form = self._get_form(request, url_kwargs)

        return self._get_response(request, form)

    def _get_response(self, request, form):
        return self._response_factory(
            request=request,
            form=form,
            response_class=self._response_class
        )

    def _response_factory(self, **kwargs):
        return kwargs['response_class'](kwargs['form'])

    def _get_form(self, request, url_kwargs):

        if self._form_class is None:
            return None

        return self._form_factory(
            request=request,
            form_class=self._form_class,
            url_kwargs=url_kwargs)

    def _form_factory(self, **kwargs):
        return kwargs['form_class']()




from django.http.response import HttpResponseBadRequest

from basement import http
from basement.views._base_view import BaseView


class ActionView(BaseView):

    _message = None

    def __init__(
            self,
            action,
            http_method='POST',
            response_class=http.JsonResponse,
            response_factory=None,
            message=None,
            **kwargs
            ):

        self._http_method = http_method

        setattr(self, http_method.lower(), self._process_request)

        self._action = action

        self._response_class = response_class

        if response_factory is not None:
            self._response_factory = response_factory

        if message is not None:
            self._message = message

        super(ActionView, self).__init__(**kwargs)

    def _process_request(self, request, **url_kwargs):

        try:
            action_result = self._action(
                request, url_kwargs, http.get_request_data(request))
        except Exception as e:
            return HttpResponseBadRequest(str(e))

        return self._get_response(request, action_result)

    def _get_response(self, request, action_result):
        return self._response_factory(
            request=request,
            action_result=action_result,
            message=self._message,
            response_class=self._response_class
        )

    def _response_factory(self, **kwargs):
        result = (kwargs.get('action_result') or {}).copy()
        result['message'] = kwargs['message']
        return kwargs['response_class'](result)




from django.apps import apps
from django.urls import path, include
from django.conf import settings

from pydoc import locate
from basement import views


def get_urlpatterns(is_db_download_enabled=False):

    result = [
        path('raise-exception/', lambda request: 1/0)
    ]

    if is_db_download_enabled:
        result.append(
            path('db/download/', views.download_db, name='download-db'))

    if apps.is_installed('ckeditor_uploader'):
        result.append(path('ckeditor/', include('ckeditor_uploader.urls')))

    for app in settings.INSTALLED_APPS:
        located_urls = locate('{}.urls.app_urls'.format(app))

        if not located_urls:
            continue

        result += located_urls

    return result



from basement import http
from basement.utils import decorate
from basement.views._request_dispatcher import RequestDispatcher


class BaseView(object):

    def __init__(self, decorators=None):
        self._decorators = decorators

    def __add__(self, other):
        return RequestDispatcher([self]) + other

    def __call__(self, request, *args, **kwargs):

        handler = getattr(
            self, request.method.lower(),
            self._handle_unsupported_method)

        handler = decorate(handler, self._decorators)

        return handler(request, *args, **kwargs)

    def _handle_unsupported_method(self, *args, **kwargs):
        raise http.UnsupportedMethod()




from django.conf import settings

from basement.services import ServiceLocator


class RequestEnvironment(object):

    def __init__(self, session, user):

        self.session = session
        self.user = user
        self.is_debug = settings.DEBUG
        self._service_locator = ServiceLocator(user=user)

    def __getattr__(self, name):
        return getattr(self._service_locator, name)


class RequestEnvironmentMiddleware(object):

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        session = getattr(request, 'session', None)
        request.env = RequestEnvironment(session, request.user)
        return self.get_response(request)




import os
import email
import imaplib

from datetime import datetime

from django.conf import settings


ATTACHMENTS_DIR = os.path.join(settings.BASE_DIR, 'tmp', 'email_attachments')

XML_CONTENT_TYPE = (
    'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet')


class GMailMessage(object):

    def __init__(self, msg_id, msg):

        self._msg_id = msg_id
        self._msg = msg

    def has_attachments(self):
        return self._msg.is_multipart()

    def get_datetime(self):

        date_str = self._msg.get('date')

        if not date_str:
            return None

        date_data = email.utils.parsedate_tz(date_str)

        if not date_data:
            return None

        return datetime.fromtimestamp(email.utils.mktime_tz(date_data))

    def get_attachments(self, content_types):

        if not self.has_attachments():
            return []

        for part in self._msg.walk():

            if part.get_content_type() in content_types:
                open(part.get_filename(), 'wb').write(
                    part.get_payload(decode=True))

            if (
                    part.get_content_maintype() == 'multipart' or
                    part.get('Content-Disposition') is None or
                    not part.get_filename()):
                continue

            file_name = part.get_filename()
            file_path = os.path.join(ATTACHMENTS_DIR, file_name)

            if not os.path.isfile(file_path):
                fp = open(file_path, 'wb')
                fp.write(part.get_payload(decode=True))
                fp.close()

            try:
                os.remove(part.get_filename())
            except Exception as e:
                print('Failed to remove temporary file: {}'.format(e))

            yield {
                'name': file_name,
                'path': file_path
            }


class GMailInbox(object):

    def __init__(self, login, password):

        imap = imaplib.IMAP4_SSL('imap.gmail.com')
        imap.login(login, password)
        imap.select('inbox')

        self._imap = imap

    def find(self, **kwargs):

        query_parts = []

        if 'from_uid' in kwargs:
            query_parts.append('UID {}:*'.format(kwargs['from_uid']))

        if 'sender' in kwargs:
            query_parts.append('FROM "{}"'.format(kwargs['sender']))

        print(query_parts)
        print(' '.join(query_parts))

        result, data = self._imap.uid('search', None, ' '.join(query_parts))

        if result != 'OK':
            raise Exception('Bad response: {}'.format(result))

        with_attachments = kwargs.get('with_attachments')

        for msg_id in data[0].split():

            result, data = self._imap.uid('fetch', msg_id, '(RFC822)')

            if result != 'OK':
                raise Exception('Can not read message: {}'.format(result))

            msg = email.message_from_bytes(data[0][1])

            message = GMailMessage(msg_id, msg)

            if (
                    with_attachments is not None and
                    with_attachments and
                    not message.has_attachments()
                ):
                continue

            yield message

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, tb):
        self.close()

    def close(self):
        self._imap.close()
        self._imap.logout()




from django.http.response import HttpResponseBadRequest

from basement.views._base_view import BaseView


class View(BaseView):

    def __init__(
            self,
            action,
            http_method,
            response_class,
            decorators=None,
            errors_with_forms=False):

        self._action = action
        self._response_class = response_class
        self._errors_with_forms = errors_with_forms

        setattr(self, http_method.lower(), self._process_request)

        super(View, self).__init__(decorators=decorators)

    def _process_request(self, request, *args, **kwargs):

        try:
            action_result = self._action(request, *args, **kwargs)
        except Exception as e:
            return HttpResponseBadRequest(str(e))

        return self._response_class(action_result)




from django.http import QueryDict


def get_request_body_querydict(request):

    if request.environ.get('CONTENT_TYPE', '').startswith('multipart'):
        raise ValueError("Multipart content type is not supported.")
    else:
        return QueryDict(request.body, encoding=request.encoding)


def get_request_data(request):
    if request.method in ('PUT', 'DELETE'):
        return get_request_body_querydict(request)
    return getattr(request, request.method)




import os
import subprocess

from datetime import datetime
from pathlib import Path

from django.conf import settings
from django.http import FileResponse
from django.contrib.admin.views.decorators import staff_member_required


@staff_member_required
def download_db(request):

    project_name = os.path.basename(settings.BASE_DIR)

    path = os.path.join(
        str(Path.home()),
        'sites-tmp',
        project_name)

    try:
        os.makedirs(path)
    except FileExistsError:
        pass

    filename = '{}-{}.sql'.format(
        project_name,
        datetime.now().strftime('%Y-%m-%d_%H-%M'))

    full_path = os.path.join(path, filename)

    db_name = settings.DATABASES['default']['NAME']

    command = 'pg_dump {} > {}'.format(db_name, full_path)

    subprocess.call(command, shell=True)

    response = FileResponse(open(full_path, 'rb'))

    os.remove(full_path)

    return response




from django.contrib import admin
from django.shortcuts import render as django_render

from django.contrib.admin.views.decorators import staff_member_required

from basement.views import render_view
from basement.utils import decorate


@staff_member_required
def render(request, template_name, context=None, status=None):

    ctx = admin.site.each_context(request)
    ctx.update(context or {})

    return django_render(request, template_name, ctx, status=status)


def admin_render_view(*args, **kwargs):

    def decorator(f):
        return decorate(f, decorators=[
            staff_member_required,
            render_view(*args, **kwargs, renderer=render)
        ])

    return decorator
